#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import ISecurityManager, ISocialUsersFolderPlugin, ISocialUser, \
    ISocialLoginConfiguration, ISocialLoginProviderConnection
from pyams_security.zmi.interfaces import ISecurityManagerMenu
from pyams_skin.interfaces import IPageHeader, IInnerPage
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION, MANAGE_SYSTEM_PERMISSION
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import DISPLAY_MODE, IDataExtractedEvent
from z3c.table.interfaces import IColumn, IValues
from zope.component.interfaces import ISite
from zope.dublincore.interfaces import IZopeDublinCore

# import packages
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.search import SearchView, SearchResultsView
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.plugin.social import SocialUsersFolder, SocialLoginProviderConnection, get_provider_info
from pyams_security.zmi.utility import SecurityManagerPluginsTable
from pyams_skin.container import ContainerView, delete_container_element
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.skin import apply_skin
from pyams_skin.table import I18nColumn, BaseTable, ActionColumn, TrashColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_skin.viewlet.toolbar import ToolbarMenuItem, ToolbarAction
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.date import format_datetime
from pyams_utils.registry import query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyams_zmi.view import AdminView
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface, Invalid

from pyams_security import _


#
# Users folder views
#

@viewlet_config(name='add-social-users-folder.menu', context=ISite, layer=IAdminLayer,
                view=SecurityManagerPluginsTable, manager=IToolbarAddingMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=50)
class SocialUsersFolderAddMenu(ToolbarMenuItem):
    """Social users folder add menu"""

    label = _("Add social users folder...")
    label_css_class = 'fa fa-fw fa-share-alt'
    url = 'add-social-users-folder.html'
    modal_target = True


@pagelet_config(name='add-social-users-folder.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class SocialUsersFolderAddForm(AdminDialogAddForm):
    """Social users folder plug-in add form"""

    title = _("System security manager")
    legend = _("Add social users folder plug-in")
    icon_css_class = 'fa fa-fw fa-share-alt'

    fields = field.Fields(ISocialUsersFolderPlugin).omit('__name__', '__parent__')
    ajax_handler = 'add-social-users-folder.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def create(self, data):
        return SocialUsersFolder()

    def add(self, plugin):
        context = query_utility(ISecurityManager)
        context[plugin.prefix] = plugin

    def nextURL(self):
        return absolute_url(self.context, self.request, 'security-manager.html')


@subscriber(IDataExtractedEvent, form_selector=SocialUsersFolderAddForm)
def handle_new_user_folder_data_extraction(event):
    """Handle new social users folder form data extraction"""
    manager = query_utility(ISecurityManager)
    prefix = event.data.get('prefix')
    if prefix in manager:
        event.form.widgets.errors += (Invalid(_("Specified prefix is already used!")),)


@view_config(name='add-social-users-folder.json', context=ISite, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class SocialUsersFolderAJAXAddForm(AJAXAddForm, SocialUsersFolderAddForm):
    """Social users folder plug-in add form, AJAX handler"""


@pagelet_config(name='properties.html', context=ISocialUsersFolderPlugin, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SocialUsersFolderEditForm(AdminDialogEditForm):
    """Social users folder plug-in edit form"""

    prefix = 'social_properties.'

    @property
    def title(self):
        return self.context.title

    legend = _("Edit social users folder plug-in properties")
    icon_css_class = 'fa fa-fw fa-share-alt'

    fields = field.Fields(ISocialUsersFolderPlugin).omit('__name__', '__parent__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(SocialUsersFolderEditForm, self).updateWidgets()
        self.widgets['prefix'].mode = DISPLAY_MODE


@view_config(name='properties.json', context=ISocialUsersFolderPlugin, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class SocialUsersFolderAJAXEditForm(AJAXEditForm, SocialUsersFolderEditForm):
    """Social users folder plug-in edit form, AJAX handler"""


#
# Social users folder search views
#

@pagelet_config(name='search.html', context=ISocialUsersFolderPlugin, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SocialUsersFolderSearchView(SearchView):
    """Social users folder search view"""
    
    def __init__(self, context, request):
        super(SocialUsersFolderSearchView, self).__init__(context, request)


@adapter_config(context=(ISocialUsersFolderPlugin, IPyAMSLayer, SocialUsersFolderSearchView), provides=IPageHeader)
class SocialUsersFolderSearchViewHeaderAdapter(DefaultPageHeaderAdapter):
    """Social users folder search view header adapter"""

    back_url = '#security-manager.html'
    back_target = '#content'
    icon_class = 'fa fa-fw fa-share-alt'
    title = _("Security manager")

    @property
    def subtitle(self):
        return self.context.title


@view_config(name='search-results.html', context=ISocialUsersFolderPlugin, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION)
class SocialUsersFolderSearchResultsView(AdminView, SearchResultsView):
    """Social users folder search results view table"""

    id = 'social_users_folder_search_table'
    title = _("Search results")
    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight datatable'}

    def __init__(self, context, request):
        super(SocialUsersFolderSearchResultsView, self).__init__(context, request)
        apply_skin(self.request, 'PyAMS admin skin')


@adapter_config(name='login', context=(ISocialUsersFolderPlugin, IAdminLayer, SocialUsersFolderSearchResultsView),
                provides=IColumn)
class IDColumn(I18nColumn, GetAttrColumn):
    """Users ID column"""

    _header = _("User ID")
    attrName = 'user_id'
    weight = 5


@adapter_config(name='name', context=(ISocialUsersFolderPlugin, IAdminLayer, SocialUsersFolderSearchResultsView),
                provides=IColumn)
class NameColumn(I18nColumn, GetAttrColumn):
    """Users name column"""

    _header = _("Name")
    attrName = 'title'
    weight = 10


@adapter_config(name='email', context=(ISocialUsersFolderPlugin, IAdminLayer, SocialUsersFolderSearchResultsView),
                provides=IColumn)
class EmailColumn(I18nColumn, GetAttrColumn):
    """Users email column"""

    _header = _("E-mail address")
    attrName = 'email'
    weight = 20


@adapter_config(name='provider_name', context=(ISocialUsersFolderPlugin, IAdminLayer, SocialUsersFolderSearchResultsView),
                provides=IColumn)
class ProviderNameColumn(I18nColumn, GetAttrColumn):
    """Users provider name column"""

    _header = _("OAuth provider name")
    attrName = 'provider_name'
    weight = 30


@adapter_config(name='registration_date', context=(ISocialUsersFolderPlugin, IAdminLayer, SocialUsersFolderSearchResultsView),
                provides=IColumn)
class RegistrationDateColumn(I18nColumn, GetAttrColumn):
    """Users registration date column"""

    _header = _("Registration date")
    weight = 40

    def getValue(self, obj):
        dc = IZopeDublinCore(obj, None)
        if dc is not None:
            return format_datetime(dc.created)
        else:
            return '--'


#
# Social users views
#

@pagelet_config(name='properties.html', context=ISocialUser, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class SocialUserEditForm(AdminDialogEditForm):
    """Social user edit form"""

    prefix = 'socialuser_properties.'

    @property
    def title(self):
        return self.context.title

    legend = _("Edit user properties")
    icon_css_class = 'fa fa-fw fa-share-alt'
    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    fields = field.Fields(ISocialUser).omit('__parent__', '__name__', 'attributes')

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(SocialUserEditForm, self).updateWidgets()
        self.widgets['user_id'].mode = DISPLAY_MODE
        self.widgets['provider_name'].mode = DISPLAY_MODE
        self.widgets['registration_date'].mode = DISPLAY_MODE


@view_config(name='properties.json', context=ISocialUser, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class SocialUserAJAXEditForm(AJAXEditForm, SocialUserEditForm):
    """Social user edit form, AJAX view"""


#
# Social providers configuration
#

@viewlet_config(name='security-manager.social.menu', context=ISite, layer=IAdminLayer,
                manager=ISecurityManagerMenu, permission=VIEW_SYSTEM_PERMISSION, weight=10)
class SecurityManagerSocialMenuItem(MenuItem):
    """Security manager social menu"""

    label = _("Social networks login")
    url = '#social-providers.html'


class SecurityManagerSocialProvidersTable(BaseTable):
    """Security manager social providers table"""

    id = 'social_providers_table'
    title = _("Configured social networks login providers")
    cssClasses = {'table': 'table table-bordered table-striped table-hover table-tight datatable'}

    @property
    def data_attributes(self):
        manager = query_utility(ISecurityManager)
        if manager is not None:
            return {
                'table': {
                    'data-ams-location': absolute_url(ISocialLoginConfiguration(manager), self.request)
                },
                'tr': {
                    'data-ams-element-name': lambda x, col: x.provider_name,
                    'data-ams-url': lambda x, col: absolute_url(x, self.request, 'properties.html'),
                    'data-toggle': 'modal'
                }
            }
        else:
            return {}


@adapter_config(name='icon', context=(Interface, IAdminLayer, SecurityManagerSocialProvidersTable), provides=IColumn)
class SecurityManagerSocialProvidersIconColumn(ActionColumn):
    """Security manager plugins icon column"""

    weight = 1

    def renderCell(self, item):
        info = get_provider_info(item.provider_name)
        return '<i class="{0}"></i>'.format(info.icon_class)


@adapter_config(name='id', context=(Interface, IAdminLayer, SecurityManagerSocialProvidersTable), provides=IColumn)
class SecurityManagerSocialProvidersIdColumn(I18nColumn, GetAttrColumn):
    """Security manager plugins name column"""

    _header = _("ID")
    cssClasses = {'th': 'action'}
    attrName = 'provider_id'
    weight = 5


@adapter_config(name='name', context=(Interface, IAdminLayer, SecurityManagerSocialProvidersTable), provides=IColumn)
class SecurityManagerSocialProvidersNameColumn(I18nColumn, GetAttrColumn):
    """Security manager plugins name column"""

    _header = _("Name")
    attrName = 'provider_name'
    weight = 10


@adapter_config(name='trash', context=(Interface, IAdminLayer, SecurityManagerSocialProvidersTable), provides=IColumn)
class SecurityManagerSocialProvidersTrashColumn(TrashColumn):
    """Security manager plugins trash column"""

    icon_hint = _("Delete provider")
    permission = MANAGE_SYSTEM_PERMISSION


@view_config(name='delete-element.json', context=ISocialLoginConfiguration, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
def delete_social_provider(request):
    """Delete social provider from security manager"""
    return delete_container_element(request, ignore_permission=True)


@adapter_config(context=(ISite, IAdminLayer, SecurityManagerSocialProvidersTable), provides=IValues)
class SecurityManagerSocialProvidersValuesAdapter(ContextRequestViewAdapter):
    """Security manager social providers values adapter"""

    @property
    def values(self):
        manager = query_utility(ISecurityManager)
        if manager is not None:
            configuration = ISocialLoginConfiguration(manager)
            return list(configuration.values())
        return ()


@pagelet_config(name='social-providers.html', context=ISite, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IInnerPage)
class SecurityManagerSocialProvidersView(AdminView, ContainerView):
    """Security manager social providers view"""

    table_class = SecurityManagerSocialProvidersTable

    def __init__(self, context, request):
        super(SecurityManagerSocialProvidersView, self).__init__(context, request)


@adapter_config(context=(ISite, IAdminLayer, SecurityManagerSocialProvidersView), provides=IPageHeader)
class SecurityManagerSocialProvidersHeaderAdapter(DefaultPageHeaderAdapter):
    """Security manager social providers header adapter"""

    back_url = '#security-manager.html'
    icon_class = 'fa fa-fw fa-share-alt'
    title = _("Security manager")
    subtitle = _("Social networks login providers")


@viewlet_config(name='security-manager.social.adding', context=ISite, view=SecurityManagerSocialProvidersTable,
                layer=IAdminLayer, manager=IWidgetTitleViewletManager, permission=MANAGE_SYSTEM_PERMISSION)
class SocialToolbarAddingsAction(ToolbarAction):
    """Security manager social toolbar adding action"""

    label = _("Add provider")
    url = "add-social-provider.html"
    modal_target = True


@pagelet_config(name='add-social-provider.html', context=ISite, layer=IPyAMSLayer, permission=MANAGE_SYSTEM_PERMISSION)
class SocialProviderAddForm(AdminDialogAddForm):
    """Social provider add form"""

    title = _("Security manager")
    legend = _("Add new social login provider")
    icon_css_class = 'fa fa-fw fa-share-alt'
    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    fields = field.Fields(ISocialLoginProviderConnection).omit('__parent__', '__name__')

    ajax_handler = 'add-social-provider.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(SocialProviderAddForm, self).updateWidgets()
        self.widgets['provider_id'].input_css_class = 'col-md-3'

    def create(self, data):
        return SocialLoginProviderConnection()

    def add(self, object):
        manager = query_utility(ISecurityManager)
        configuration = ISocialLoginConfiguration(manager)
        configuration[object.provider_name] = object


@subscriber(IDataExtractedEvent, form_selector=SocialProviderAddForm)
def handle_new_social_provider_data_extraction(event):
    """Handle new social provider data extraction"""
    data = event.data
    manager = query_utility(ISecurityManager)
    configuration = ISocialLoginConfiguration(manager)
    if data.get('provider_name') in configuration:
        event.form.widgets.errors += (Invalid(_("This provider is already defined!")),)
    for provider in configuration.values():
        if data.get('provider_id') == provider.provider_id:
            event.form.widgets.errors += (Invalid(_("This provider ID is already used!")),)
            break


@view_config(name='add-social-provider.json', context=ISite, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class SocialProviderAJAXAddForm(AJAXAddForm, SocialProviderAddForm):
    """Social provider add form, AJAX view"""

    def get_ajax_output(self, changes):
        translate = self.request.localizer.translate
        return {
            'status': 'reload',
            'location': absolute_url(self.context, self.request, 'social-providers.html'),
            'message': translate(_("Social provider was created successfully"))
        }


@pagelet_config(name='properties.html', context=ISocialLoginProviderConnection, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SocialProviderEditForm(AdminDialogEditForm):
    """Social provider edit form"""

    prefix = 'provider_properties.'

    title = _("Security manager")
    legend = _("Edit social login provider properties")
    icon_css_class = 'fa fa-fw fa-share-alt'
    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    fields = field.Fields(ISocialLoginProviderConnection).omit('__parent__', '__name__')

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(SocialProviderEditForm, self).updateWidgets()
        self.widgets['provider_name'].mode = DISPLAY_MODE
        self.widgets['provider_id'].mode = DISPLAY_MODE


@view_config(name='properties.json', context=ISocialLoginProviderConnection, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class SocialProviderAJAXEditForm(AJAXEditForm, SocialProviderEditForm):
    """Social provider edit form, AJAX view"""
